/*******************************************************************************
**
**  Module  : tpro_func.h
**  Date    : 04/03/06
**  Purpose : This driver provides an interface to the TPRO-PCI
**
**  Copyright(C) 2006 Spectracom Corporation. All Rights Reserved.
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
**
********************************************************************************
**
**     04/03/06 Updated from Linux (2.4.24) to Linux (2.6.11.12)
**
*******************************************************************************/
#ifndef _defined_TPRO_FUNC_
#define _defined_TPRO_FUNC_

/*******************************************************************************
**          Hardware Access Macros
*******************************************************************************/

#define PEEK_REG(base, offs) \
            inw ((unsigned short)((base) + (offs)))

#define PEEK_REG_32(base, offs)                    \
            inl ((unsigned short)((base) + (offs)))

#define PEEK_REG_CPCI(base, offs)                                           \
            (ioread32((void*)base + (offs)))

#define POKE_REG(base, offs, val) \
            outw ((unsigned short) (val), (unsigned short)((base) + (offs)))

#define POKE_REG_32(base, offs, val)                                   \
            outl ((unsigned short) (val), (unsigned short)((base) + (offs)))

#define POKE_REG_CPCI(base, offs, val)                                      \
            (iowrite32((val), (void*)base + (offs)))

#define UPWORD_HI_NIBBLE(x)     (((x) & 0xF0000000) >> 28)
#define UPWORD_MID_HI_NIBBLE(x) (((x) & 0x0F000000) >> 24)
#define UPWORD_MID_LO_NIBBLE(x) (((x) & 0x00F00000) >> 20)
#define UPWORD_LO_NIBBLE(x)     (((x) & 0x000F0000) >> 16)
#define HI_NIBBLE(x)            (((x) & 0x0000F000) >> 12)
#define MID_HI_NIBBLE(x)        (((x) & 0x00000F00) >> 8)
#define MID_LO_NIBBLE(x)        (((x) & 0x000000F0) >> 4)
#define LO_NIBBLE(x)            (((x) & 0x0000000F) >> 0)
#define CHAR_TO_DEC(val)      ( (val) - '0')

/*******************************************************************************
**          Defines
*******************************************************************************/
#define FIFO_RESPONSE_LENGTH  (10)

/*
** Interrupt control flags
*/
#define DISABLE_INTERRUPTS   (0x00)
#define FIFO_IRQ_ENABLE      (0x80)
#define FIFO_IRQ_DISABLE     (0x7F)
#define MATCH_IRQ_ENABLE     (0x40)
#define MATCH_IRQ_DISABLE    (0xBF)
#define HEART_IRQ_ENABLE     (0x20)
#define HEART_IRQ_DISABLE    (0xDF)

/*
**  TPRO-PCI REGISTER LOCATIONS 16bit offsets
*/
#define FIFO_INT_CTRL_REG    (0x00)
#define CMD_STAT_REG         (0x02)
#define RESET_DEASSERT_REG   (0x04)
#define RESET_ASSERT_REG     (0x06)
#define CLK_DAYS_REG         (0x08)
#define CLK_HRS_MINS_REG     (0x0A)
#define CLK_SECS_MSECS_REG   (0x0C)
#define CLK_MSECS_USECS_REG  (0x0E)
#define CLR_FLAGS_REG        (0x10)
#define SIM_TAG_REG          (0x12)

/*
**  TPRO-PCI REGISTER LOCATIONS 32bit offsets
*/
#define FIFO_INT_CTRL_REG_32  (0x00)
#define CMD_STAT_REG_32       (0x04)
#define RESET_DEASSERT_REG_32 (0x08)
#define RESET_ASSERT_REG_32   (0x0C)
#define TIME_REG_LOW_32       (0x10)
#define TIME_REG_HIGH_32      (0x14)
#define CLR_FLAGS_REG_32      (0x18)
#define SIM_TAG_REG_32        (0x1C)
#define FPGA_REV_32           (0x20)

/*******************************************************************************
**          Public Function Prototypes 16bit registers
*******************************************************************************/
unsigned char tproGetAltitude      (tpropci_dev_t *hw, AltObj *Altp);
unsigned char tproGetDate          (tpropci_dev_t *hw, DateObj *Datep);
unsigned char tproGetEvent         (tpropci_dev_t *hw, WaitObj *Waitp);
unsigned char tproGetFirmware      (tpropci_dev_t *hw, unsigned char *firm);
unsigned char tproInitializeBoard  (tpropci_dev_t *hw);
unsigned char tproGetLatitude      (tpropci_dev_t *hw, LatObj *Latp);
unsigned char tproGetLongitude     (tpropci_dev_t *hw, LatObj *Longp);
unsigned char tproGetSatInfo       (tpropci_dev_t *hw, SatObj *Satp);
unsigned char tproGetTime          (tpropci_dev_t *hw, TimeObj *Timep);
unsigned char tproGetNtpTime       (tpropci_dev_t *hw, NtpTimeObj *Timep);
unsigned char tproResetFirmware    (tpropci_dev_t *hw);
unsigned char tproSetHeartbeat     (tpropci_dev_t *hw, HeartObj *Heartp);
unsigned char tproSetMatchTime     (tpropci_dev_t *hw, MatchObj *Matchp);
unsigned char tproSetPropDelayCorr (tpropci_dev_t *hw, int *microseconds);
unsigned char tproSetTime          (tpropci_dev_t *hw, TimeObj *Timep);
unsigned char tproSetYear          (tpropci_dev_t *hw, unsigned short *year);
unsigned char tproSimEvent         (tpropci_dev_t *hw);
unsigned char tproSynchControl     (tpropci_dev_t *hw, unsigned char *enbp);
unsigned char tproSynchStatus      (tpropci_dev_t *hw, unsigned char *stat);
unsigned char tproPeek             (tpropci_dev_t *hw, MemObj *Mem);
unsigned char tproPoke             (tpropci_dev_t *hw, MemObj *Mem);
unsigned char tproGetFpgaVersion   (tpropci_dev_t *hw, unsigned char *fpgaver);
unsigned char tproToggleInterrupt  (tpropci_dev_t *hw, unsigned char mask);
unsigned char tproSetOsc           (tpropci_dev_t *hw, unsigned char *freq);
unsigned char tproISR              (tpropci_dev_t *hw, int *irq);

/*******************************************************************************
**          Public Function Prototypes 32bit registers
*******************************************************************************/
unsigned char tproGetAltitude32      (tpropci_dev_t *hw, AltObj *Altp);
unsigned char tproGetDate32          (tpropci_dev_t *hw, DateObj *Datep);
unsigned char tproGetEvent32         (tpropci_dev_t *hw, WaitObj *Waitp);
unsigned char tproGetFirmware32      (tpropci_dev_t *hw, unsigned char *firm);
unsigned char tproInitializeBoard32  (tpropci_dev_t *hw);
unsigned char tproGetLatitude32      (tpropci_dev_t *hw, LatObj *Latp);
unsigned char tproGetLongitude32     (tpropci_dev_t *hw, LatObj *Longp);
unsigned char tproGetSatInfo32       (tpropci_dev_t *hw, SatObj *Satp);
unsigned char tproGetTime32          (tpropci_dev_t *hw, TimeObj *Timep);
unsigned char tproGetNtpTime32       (tpropci_dev_t *hw, NtpTimeObj *Timep);
unsigned char tproResetFirmware32    (tpropci_dev_t *hw);
unsigned char tproSetHeartbeat32     (tpropci_dev_t *hw, HeartObj *Heartp);
unsigned char tproSetMatchTime32     (tpropci_dev_t *hw, MatchObj *Matchp);
unsigned char tproSetPropDelayCorr32 (tpropci_dev_t *hw, int *microseconds);
unsigned char tproSetTime32          (tpropci_dev_t *hw, TimeObj *Timep);
unsigned char tproSetYear32          (tpropci_dev_t *hw, unsigned short *year);
unsigned char tproSimEvent32         (tpropci_dev_t *hw);
unsigned char tproSynchControl32     (tpropci_dev_t *hw, unsigned char *enbp);
unsigned char tproSynchStatus32      (tpropci_dev_t *hw, unsigned char *stat);
unsigned char tproPeek32             (tpropci_dev_t *hw, MemObj *Mem);
unsigned char tproPoke32             (tpropci_dev_t *hw, MemObj *Mem);
unsigned char tproGetFpgaVersion32   (tpropci_dev_t *hw, unsigned char *fpgaver);
unsigned char tproToggleInterrupt32  (tpropci_dev_t *hw, unsigned char mask);
unsigned char tproSetOsc32           (tpropci_dev_t *hw, unsigned char *freq);
unsigned char tproISR32              (tpropci_dev_t *hw, int *irq);

/*******************************************************************************
**          Public Function Prototypes cPCI registers
*******************************************************************************/
unsigned char tproGetAltitudeCPCI      (tpropci_dev_t *hw, AltObj *Altp);
unsigned char tproGetDateCPCI          (tpropci_dev_t *hw, DateObj *Datep);
unsigned char tproGetEventCPCI         (tpropci_dev_t *hw, WaitObj *Waitp);
unsigned char tproGetFirmwareCPCI      (tpropci_dev_t *hw, unsigned char *firm);
unsigned char tproInitializeBoardCPCI  (tpropci_dev_t *hw);
unsigned char tproGetLatitudeCPCI      (tpropci_dev_t *hw, LatObj *Latp);
unsigned char tproGetLongitudeCPCI     (tpropci_dev_t *hw, LatObj *Longp);
unsigned char tproGetSatInfoCPCI       (tpropci_dev_t *hw, SatObj *Satp);
unsigned char tproGetTimeCPCI          (tpropci_dev_t *hw, TimeObj *Timep);
unsigned char tproGetNtpTimeCPCI       (tpropci_dev_t *hw, NtpTimeObj *Timep);
unsigned char tproResetFirmwareCPCI    (tpropci_dev_t *hw);
unsigned char tproSetHeartbeatCPCI     (tpropci_dev_t *hw, HeartObj *Heartp);
unsigned char tproSetMatchTimeCPCI     (tpropci_dev_t *hw, MatchObj *Matchp);
unsigned char tproSetPropDelayCorrCPCI (tpropci_dev_t *hw, int *microseconds);
unsigned char tproSetTimeCPCI          (tpropci_dev_t *hw, TimeObj *Timep);
unsigned char tproSetYearCPCI          (tpropci_dev_t *hw, unsigned short *year);
unsigned char tproSimEventCPCI         (tpropci_dev_t *hw);
unsigned char tproSynchControlCPCI     (tpropci_dev_t *hw, unsigned char *enbp);
unsigned char tproSynchStatusCPCI      (tpropci_dev_t *hw, unsigned char *stat);
unsigned char tproPeekCPCI             (tpropci_dev_t *hw, MemObj *Mem);
unsigned char tproPokeCPCI             (tpropci_dev_t *hw, MemObj *Mem);
unsigned char tproGetFpgaVersionCPCI   (tpropci_dev_t *hw, unsigned char *fpgaver);
unsigned char tproToggleInterruptCPCI  (tpropci_dev_t *hw, unsigned char mask);
unsigned char tproSetOscCPCI           (tpropci_dev_t *hw, unsigned char *freq);
unsigned char tproISRCPCI              (tpropci_dev_t *hw, int *irq);

/*******************************************************************************
**          Structure definiation for the Function pointer table.
*******************************************************************************/
typedef struct funcTable_s
{
    unsigned char (*getAltitude)      (tpropci_dev_t *hw, AltObj *Altp);
    unsigned char (*getDate)          (tpropci_dev_t *hw, DateObj *Datep);
    unsigned char (*getEvent)         (tpropci_dev_t *hw, WaitObj *Waitp);
    unsigned char (*getFirmware)      (tpropci_dev_t *hw, unsigned char *firm);
    unsigned char (*InitializeBoard)  (tpropci_dev_t *hw);
    unsigned char (*getLatitude)      (tpropci_dev_t *hw, LatObj *Latp);
    unsigned char (*getLongitude)     (tpropci_dev_t *hw, LatObj *Longp);
    unsigned char (*getSatInfo)       (tpropci_dev_t *hw, SatObj *Satp);
    unsigned char (*getTime)          (tpropci_dev_t *hw, TimeObj *Timep);
    unsigned char (*getNtpTime)       (tpropci_dev_t *hw, NtpTimeObj *Timep);
    unsigned char (*resetFirmware)    (tpropci_dev_t *hw);
    unsigned char (*setHeartbeat)     (tpropci_dev_t *hw, HeartObj *Heartp);
    unsigned char (*setMatchTime)     (tpropci_dev_t *hw, MatchObj *Matchp);
    unsigned char (*setPropDelayCorr) (tpropci_dev_t *hw, int *microseconds);
    unsigned char (*setTime)          (tpropci_dev_t *hw, TimeObj *Timep);
    unsigned char (*setYear)          (tpropci_dev_t *hw, unsigned short *year);
    unsigned char (*simEvent)         (tpropci_dev_t *hw);
    unsigned char (*synchControl)     (tpropci_dev_t *hw, unsigned char *enbp);
    unsigned char (*synchStatus)      (tpropci_dev_t *hw, unsigned char *stat);
    unsigned char (*peek)             (tpropci_dev_t *hw, MemObj *Mem);
    unsigned char (*poke)             (tpropci_dev_t *hw, MemObj *Mem);
    unsigned char (*getFpgaVersion)   (tpropci_dev_t *hw, unsigned char *fpgaVer);
    unsigned char (*toggleInterrupt)  (tpropci_dev_t *hw, unsigned char mask);
    unsigned char (*setOsc)           (tpropci_dev_t *hw, unsigned char *freq);
    unsigned char (*isr)              (tpropci_dev_t *hw, int *irq);

    /* define register offsets that are used outside the board specific files */
    int cmdStatOffset;
    int fifoIntCtrlOffset;
    int clrFlagsOffset;

} FuncTable_t;

#endif  /* _defined_TPRO_FUNC_ */
