#include <stdio.h>
#include <tpro.h>
#include <ddtpro.h>


// heartbeat constants
#define OPTION_HI_RATE		(0)		// hi oscillator option
#define OPTION_LO_RATE  	(1)		// low oscillator option
#define OPTION_VAR_RATE  	(2)		// variable option

#define HEART_10MHZ_PER     (10000000.0)
#define HEART_3MHZ_PER      (3000000.0)
#define HEART_1MHZ_PER      (1000000.0)
#define HEART_1KHZ_PER      (1000.0)


/*==========================================================================
  TPRO_setHeartbeat
==========================================================================*/

unsigned char TPRO_setHeartbeat (TPRO_BoardObj *hnd, TPRO_HeartObj *Heartp)
{
  unsigned long length;
  unsigned char option;
  int err;

  HeartObj Heartbeat;


  // make sure we have valid handle
  if (hnd == NULL) {
    return (TPRO_HANDLE_ERR);
  }

  /**
  ***  determine how to set heartbeat based on board options
  **/

  if ((hnd->options == 0x9052) || (hnd->options == 0x9053) ||
      (hnd->options == 0x9072) ||
      (hnd->options == 0x9152) || (hnd->options == 0x9153) ||
      (hnd->options == 0x9172) ) {
    option = OPTION_LO_RATE;
  }
  else if ((hnd->options == 0x7000) || 
           (hnd->options == 0x7100) ||
           (hnd->options == 0x7200) ||
           (hnd->options == 0x7300)) {
    option = OPTION_VAR_RATE;
  }
  else {
    option = OPTION_HI_RATE;
  }


  /**
  ***  calculate divide number
  **/
  if (option == OPTION_VAR_RATE) {

    switch (Heartp->outputType)
    {
        case CLK_10MHZ:
            Heartbeat.divNumber  = 
                    (unsigned short)(65536 - (HEART_10MHZ_PER / Heartp->frequency));
            Heartbeat.outputType = CLK_10MHZ;
            if (Heartbeat.divNumber > 0xFFFE) return (TPRO_FREQ_ERR);

            break;

        case CLK_3MHZ:
            Heartbeat.divNumber  = 
                    (unsigned short)(65536 - (HEART_3MHZ_PER / Heartp->frequency));
            Heartbeat.outputType = CLK_3MHZ;
            if ((Heartbeat.divNumber < 0x3) &&
                     (Heartbeat.divNumber > 0xFFFC)) return (TPRO_FREQ_ERR);
            if (Heartbeat.divNumber % 3) return (TPRO_FREQ_ERR);

            break;

        case CLK_1MHZ:
            Heartbeat.divNumber  = 
                    (unsigned short)(65536 - (HEART_1MHZ_PER / Heartp->frequency));
            Heartbeat.outputType = CLK_1MHZ;
            if (Heartbeat.divNumber > 0xFFFE) return (TPRO_FREQ_ERR);

            break;

        case CLK_1KHZ:
            Heartbeat.divNumber  = 
                    (unsigned short)(65536 - (HEART_1KHZ_PER / Heartp->frequency));
            Heartbeat.outputType = CLK_1KHZ;
            if (Heartbeat.divNumber > 0xFFFE) return (TPRO_FREQ_ERR);

            break;

        // unsupported clock selection
        default:
            return (TPRO_FREQ_ERR);
    }
  }
  else
  {
    if (option == OPTION_HI_RATE) {

      // calculate
      Heartbeat.divNumber = (unsigned short) (3000000 / Heartp->frequency);

      // verify valid divide number
      if ( (Heartp->frequency > 1000000) || (Heartp->frequency < 45.7771) ) return (TPRO_FREQ_ERR);
      if (Heartbeat.divNumber % 3) return (TPRO_FREQ_ERR);
    }

    else {

      // calculate
      Heartbeat.divNumber = (unsigned short) (1000 / Heartp->frequency);

      // verify valid divide number
      if ( (Heartp->frequency > 500) || (Heartp->frequency < 0.0152593) ) return (TPRO_FREQ_ERR);
      if (Heartbeat.divNumber < 2) return (TPRO_FREQ_ERR);
    }


    /**
    ***  fill-in working context
    **/

    Heartbeat.signalType = Heartp->signalType;
    Heartbeat.outputType = Heartp->outputType;
  }


  /**
  ***  create I/O control
  **/

  err = ioctl(hnd->file_descriptor, IOCTL_TPRO_SET_HEARTBEAT, &Heartbeat);
  err = (err < 0) ? TPRO_COMM_ERR : TPRO_SUCCESS;

  return (err);
}
