#include <stdio.h>
#include <tpro.h>
#include <ddtpro.h>


/*==========================================================================
  TPRO_setTime
==========================================================================*/

unsigned char TPRO_setTime (TPRO_BoardObj *hnd, TPRO_TimeObj *Timep)
{
  unsigned char syncOption = 0;
  unsigned long length;
  int err;

  TimeObj TimeContext;


  // make sure we have valid handle
  if (hnd == NULL) {
    return (TPRO_HANDLE_ERR);
  }

  /**
  ***  set time option based on board options
  **/

  if ((hnd->options == 0x9051) || (hnd->options == 0x9053) ||
      (hnd->options == 0x9151) || (hnd->options == 0x9153) )
  {
    syncOption = 1;
  }


  /**
  ***  verify valid time
  **/

  if (Timep->days    > 366) return (TPRO_DAY_PARM_ERR);
  if (Timep->hours   > 23 ) return (TPRO_HOUR_PARM_ERR);
  if (Timep->minutes > 59 ) return (TPRO_MIN_PARM_ERR);
  if (Timep->seconds > 59 ) return (TPRO_SEC_PARM_ERR);

  /** Compact PCI / PMC **/
  if ((hnd->options == 0x7000) || 
      (hnd->options == 0x7100) ||
      (hnd->options == 0x7200) ||
      (hnd->options == 0x7300)) {
    if ((Timep->year < 1990) || (Timep->year > 2999)) return (TPRO_YEAR_PARM_ERR);
    TimeContext.year = Timep->year;

  }

  /**
  ***  fill-in working context
  **/

  TimeContext.syncOption = syncOption;

  TimeContext.days    = Timep->days;
  TimeContext.hours   = Timep->hours;
  TimeContext.minutes = Timep->minutes;
  TimeContext.seconds = Timep->seconds;


  /**
  ***  create I/O control returns nonzero on success
  **/

  err = ioctl(hnd->file_descriptor, IOCTL_TPRO_SET_TIME, &TimeContext);
  err = (err < 0) ? TPRO_COMM_ERR : TPRO_SUCCESS;

  return (err);
}
