#include <stdlib.h>
#include <stdio.h>

#include <tpro.h>

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            retVal;
    TPRO_BoardObj *pBoard;
    TPRO_DateObj   tproDate;
    unsigned char  rv;

    retVal = 0;
    
    /* If invalid number of arguments */
    if (argc != 2) 
    {
         printf (" Usage: GetDate <device index>\n");
         return (1);
    }
    
    /* Construct the TSAT/TPRO device name */
    sprintf(deviceName, "/dev/tpropci%d", atoi(argv[1]));
    
    /* Open the indexed TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf (" Could Not Open '%s'!! [%d]\n", deviceName, rv);
        return (1);
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Get the current TPRO/TSAT date */
    rv = TPRO_getDate (pBoard, &tproDate);
    
    /* If unable to retrieve the current TPRO/TSAT date... */
    if (rv != TPRO_SUCCESS) 
    {
        printf (" Could Not Retrieve Date from Board [%d]!!\n", rv);    
        retVal = 1;
    }
    else
    {
        /* Display the current TPRO/TSAT date */
        printf(" Device Name: %s\n", deviceName);
        printf(" %d-%d-%d\n", tproDate.month, tproDate.day, tproDate.year);
    }
    
    /* Close the TPRO/TSAT device */
    rv = TPRO_close (pBoard);

    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf (" Could Not Close Board!! [%d]\n", rv);
        retVal = 1;
    }

    return (retVal);
    
} /* End - main() */
