#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>

#include <tpro.h>

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            retVal;
    TPRO_BoardObj *pBoard;
    TPRO_TimeObj   tproTime;
    double         *timeArray;
    unsigned char  rv;
    int            loopCntr;
    int            histogram[101];
    time_t         start,end;
    int            userTimeMode = 0;

    retVal = 0;
    
    /* If invalid number of arguments... */
    if (argc < 2)
    {
        printf(" Usage: GetTimePerf <device index> {-u}\n");
        printf("        <device index> is a number in the range 0 - 4\n");
        printf("        -u measure time in user space\n");
        return (1);
    }

    else if (argc == 2)
    {
        if (!(((atoi(argv[1]) == 0) && (strcmp(argv[1],"0") == 0)) || (atoi(argv[1]) > 0)))
        {
            printf("(2)Usage: GetTimePerf <device index> {-u} (%s)\n",argv[1]);
            printf("        <device index> is a number in the range 0 - 4\n");
            printf("        -u measure time in user space\n");
            return (1);
        }
    }

    else
    {
        if (!(((atoi(argv[1]) == 0) && (strcmp(argv[1],"0") == 0)) || (atoi(argv[1]) > 0)))
        {
            printf("Usage: GetTimePerf <device index> {-u}\n");
            printf("        <device index> is a number in the range 0 - 4\n");
            printf("        -u measure time in user space\n");
            return (1);
        }

        if (strcmp(argv[2], "-u") == 0)
        {
            userTimeMode = 1;
        }
        else
        {
            printf("Usage: GetTimePerf <device index> {-u}\n");
            printf("        <device index> is a number in the range 0 - 4\n");
            printf("        -u measure time in user space\n");
            return (1);
        }
    }
    
    /* Construct the TPRO/TSAT device name */
    sprintf(deviceName, "/dev/tpropci%d", atoi(argv[1]));
    
    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf (" Could Not Open '%s'!! [%d]\n", deviceName, rv);
        return (1);
    }

    timeArray = (double*)malloc(sizeof(double)*1000000);
    if (timeArray == NULL)
    {
      printf ("could not allocate timeArray\n");
      return (1);
    }

    if (userTimeMode == 1)
    {
        time(&start);
        
        for(loopCntr = 0; loopCntr < 100000000; loopCntr++)
        {
            /* Get the TPRO/TSAT current time */
            rv = TPRO_getTime(pBoard, &tproTime);
        }

        time(&end);
    }

    else
    {
        for(loopCntr = 0; loopCntr < 1000000; loopCntr++)
        {
            /* Get the TPRO/TSAT current time */
            rv = TPRO_getTime(pBoard, &tproTime);
            timeArray[loopCntr] = tproTime.secsDouble;
        }
    }
    
    /* If unable to get the TPRO/TSAT current time... */
    if (rv != TPRO_SUCCESS) 
    {
        printf (" Could not retrieve time from '%s'!! [%d]\n", deviceName, rv);
        retVal = 1;
    }
    else
    {

        /* Display the TPRO/TSAT delat time */
        double delta;
        double min = 10.0;
        double max = 0.0;
        double ave = 0.0;

        for (loopCntr = 0; loopCntr < 101; loopCntr++)
            histogram[loopCntr] = 0;
        

        for (loopCntr = 0; loopCntr < (1000000-1); loopCntr++)
        {
            if ((timeArray[loopCntr+1] - timeArray[loopCntr]) > 0)
            {
                delta = timeArray[loopCntr+1] - timeArray[loopCntr];
            }
            else
            {
                delta = 60 - (timeArray[loopCntr] - timeArray[loopCntr+1]);
            }

            if (delta < min)
            {
                min = delta;
            }

            if (delta > max)
            {
                max = delta;
            }

            ave += delta;

            if ((delta * 1000000) < 100)
            {
                histogram[(int)(delta * 1000000)]++;
            }
            else
            {
                histogram[100]++;
            }
        }

        // Account for the fact the sometimes the first element is
        // larger than the last element, due to rollover.
        if ((timeArray[999999] - timeArray[0]) > 0)
        {
            delta = timeArray[999999] - timeArray[0];
        }
        else
        {
            delta = 60.0 - (timeArray[0] - timeArray[999999]);
        }
        printf ("\nGetTime averaged over 1,000,000 queries\n");
        printf (" Delta time   : %f\n", delta);
        printf (" Max interval : %f(%f)(%d)\n", max,(max*1000000), (int)(max*1000000));
        printf (" Min interval : %f(%f)(%d)\n", min,(min*1000000),(int)(min*1000000));
        printf (" Ave interval : %f\n", ave/1000000);
        printf (" User Time    : %f\n",difftime(end,start));

        for (loopCntr = 0; loopCntr < 101; loopCntr++)
        {
            if (histogram[loopCntr] > 0)
            {
                printf("Histogram[%d] = %d (%f%%)\n",
                       loopCntr,
                       histogram[loopCntr],
                       (histogram[loopCntr]/1000000.0)*100.0);
            }
        }
        
    }

    /* Close the TPRO/TSAT device */
    rv = TPRO_close(pBoard);
    
    free(timeArray);

    /* If unable to close the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf (" Could Not Close Board!! [%d]\n", rv);
        retVal = 1;
    }

    return (retVal);

} /* End - main() */
