#include <stdio.h>
#include <stdlib.h>

#include <tpro.h>

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            retVal;
    TPRO_BoardObj *pBoard;
    TPRO_HeartObj  tproHeart = {0};
    unsigned char  rv;
    
    retVal = 0;

    if (argc < 2)
    {
        printf(" Usage: SetHeart <device index> <HEARTBEAT INFO>\n");
        return (1);
    }

    /* Construct the TPRO/TSAT device name */
    sprintf(deviceName, "/dev/tpropci%d", atoi(argv[1]));

    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Open Board '%s'!! [%d]\n", deviceName, rv);
        return (1);
    }

    // If Compact PCI / PMC and invalid number of arguments
    if ((pBoard->options == 0x7000) ||
        (pBoard->options == 0x7100) ||
        (pBoard->options == 0x7200) ||
        (pBoard->options == 0x7300))
    {
        if (argc != 4)
        {
            printf(" Usage: SetHeart <device index> <frequency in Hz> <clock rate>\n");

            /* Close the TPRO/TSAT device */
            rv = TPRO_close(pBoard);
    
            /* If unable to close the TPRO/TSAT device */
            if (rv != TPRO_SUCCESS) 
            {
                printf(" Could Not Close Board!! [%d]\n", rv);
            }

            return (1);
        }
    }
    else
    {
        /* Else if invalid number of arguments... */
        if (argc != 3) 
        {
            printf(" Usage: SetHeart <device index> <frequency in Hz>\n");

            /* Close the TPRO/TSAT device */
            rv = TPRO_close(pBoard);
    
            /* If unable to close the TPRO/TSAT device */
            if (rv != TPRO_SUCCESS) 
            {
                printf(" Could Not Close Board!! [%d]\n", rv);
            }

            return (1);
        }
    }

    /* Construct the TPRO/TSAT device name */
    sprintf(deviceName, "/dev/tpropci%d", atoi(argv[1]));
    
    /*
    ** Initialize heart object 
    */
    if ((pBoard->options == 0x7000) ||
        (pBoard->options == 0x7100) ||
        (pBoard->options == 0x7200) ||
        (pBoard->options == 0x7300))
    {
        tproHeart.outputType = (unsigned char)atoi(argv[3]);
        tproHeart.frequency  = atof(argv[2]);
    }
    else
    {
        tproHeart.signalType = SIG_PULSE;       // Ignored for cPCI/PMC
        tproHeart.outputType = SIG_JAM;         // Ignored for cPCI/PMC
        tproHeart.frequency  = atof(argv[2]);
    };
    
    /* If unable to open the TPRO/TSAT device... 
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Open Board!! [%d]\n", rv);
        return (1);
    }
*/

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Set the TPRO/TSAT heartbeat */
    rv = TPRO_setHeartbeat(pBoard, &tproHeart);
    
    /* If unable to set the TPRO/TSAT heartbeat... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could not set heartbeat!! [%d]\n", rv);
        retVal = 1;
    }

    /* Close the TPRO/TSAT device */
    rv = TPRO_close(pBoard);
    
    /* If unable to close the TPRO/TSAT device ... */
    if (rv != TPRO_SUCCESS) 
    {
       printf(" Could Not Close Board!! [%d]\n", rv);
       retVal = 1;
    }

    return (retVal);
    
} /* End - main() */
