#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <tpro.h>

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            ticks;
    int            retVal;
    float          pulsewidth = .00100 ;
    TPRO_BoardObj *pBoard;
    TPRO_TimeObj   tproTime;
    TPRO_MatchObj  tproMatch;
 
    unsigned int      set_seconds;
    unsigned char   set_minutes;
    unsigned char   set_hours;
    unsigned short  set_days;

    unsigned char  rv;
    
    retVal = 0;

    /* If invalid number of arguments... */
    if (argc != 2) 
    {
        printf( "Usage: WaitMatch <device index>\n");
        return (1);
    }

    /* Construct the TPRO/TSAT device name */
    sprintf(deviceName, "/dev/tpropci%d", atoi(argv[1]));
    
    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf( "Could Not Open Board '%s'!! [%d]\n", deviceName, rv);
        return (1);
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Get the TPRO/TSAT current time */
    rv = TPRO_getTime(pBoard, &tproTime);
    
    /* If unable to get the TPRO/TSAT current time */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could not retrieve time!! [%d]\n", rv);
        (void)TPRO_close(pBoard);
        return (1);
    }
 
    set_days         = tproTime.days;
    set_hours        = tproTime.hours;
    set_minutes      = tproTime.minutes;
    set_seconds      = tproTime.secsDouble;

    printf("   Current time is: %03d:%02d:%02d:%d\n", set_days,
                                                      set_hours, 
                                                      set_minutes, 
                                                      set_seconds);
   /*
    ** Setup match START time
    */
    tproMatch.matchType    = MATCH_TIME_START;
    tproMatch.days         = set_days;
    tproMatch.hours        = set_hours;
    tproMatch.minutes      = set_minutes;
    tproMatch.seconds      = set_seconds + 5.000;
    
    if (tproMatch.seconds >= 60.00) 
    {
      tproMatch.seconds -= 60.00;
      tproMatch.minutes++;
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Set the TPRO/TSAT Match Start time */
    rv = TPRO_setMatchTime(pBoard, &tproMatch);

    
    /* If unable to set the TPRO/TSAT match time... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could not set match time!! [%d]\n", rv);
        (void)TPRO_close(pBoard);
        return (1);
    }

    printf("   Match time is:   %03d:%02d:%02d:%f\n", tproMatch.days,
                                                      tproMatch.hours, 
                                                      tproMatch.minutes, 
                                                      tproMatch.seconds);


    /*
    ** Setup match STOP time
    */

    tproMatch.matchType    = MATCH_TIME_STOP;
    tproMatch.days         = set_days;
    tproMatch.hours        = set_hours;
    tproMatch.minutes      = set_minutes;
    tproMatch.seconds      = set_seconds + 5.000 + pulsewidth ; 
    
    if (tproMatch.seconds >= 60.00) 
    {
      tproMatch.seconds -= 60.00;
      tproMatch.minutes++;
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Set the TPRO/TSAT Match Stop time */
    rv = TPRO_setMatchTime(pBoard, &tproMatch);
    
    /* If unable to set the TPRO/TSAT match time... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could not set match time!! [%d]\n", rv);
        (void)TPRO_close(pBoard);
        return (1);
    }



    /*
    **  wait for match time
    */
    printf("\n   Waiting for match time....\n");
    ticks = 10 * 1000;  /** 10 seconds **/
    rv = TPRO_waitMatch (pBoard, &ticks);

    switch (rv) {

    case TPRO_SUCCESS: 
      printf("   Received Match Time Interrupt!!\n\n");
      break;

    case TPRO_TIMEOUT_ERR:
      printf("   Timeout Receiving Match Time!!\n\n");
      retVal = 1;
      break;

    case TPRO_COMM_ERR:
      printf("   Error Communicating with Timing Board!!\n\n");
      retVal = 1;
      break;
    }





    /* Close the TPRO/TSAT device */
    rv = TPRO_close(pBoard);
    
    /* If unable close the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Close Board!! [%d]\n", rv);
        retVal = 1;
    }

    return (retVal);
    
} /* End - main() */
